--[[
 AutoSummon module for MiniPet
]]--
--[[
Copyright (c) 2010-2012, LordFarlander
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
]]--

-- $Revision: 262 $
-- $Date: 2012-12-02 03:04:01 +0000 (Sun, 02 Dec 2012) $

local L = LibStub( "AceLocale-3.0" ):GetLocale( "MiniPet autosummon Module" )

local SpecialEvents_Aura = LibStub( "LibLordFarlander-SpecialEvents-Aura-4.0" )
if( not SpecialEvents_Aura ) then
    return
end--if

if( not LibStub( "LibLordFarlander-Randomizer-2.0", true ) ) then
    return
end--if

local moduleName = "AutoSummon"

local AutoSummon = {
    displayname = L["Auto Summon"],
    settings = {
        [L["autosummon"]] = {
            name = L["Auto summon"],
            type = "toggle",
            arg = {
                DescriptionString = L["Toggles whether companions that are learned should be automatically summoned any time they are not out."],
                VariableName = "AutoSummon",
                Default = false,
            },
            desc = MiniPet.BooleanOptionDescription,
            get = "GetGenericOption",
            set = "SetGenericOption",
        },
        [L["pvpSummon"]] = {
            type = "toggle",
            name = L["Summon in PvP areas"],
            arg = {
                DescriptionString = L["Toggles whether companions will be automatically summoned when in a PvP area (battleground, arena, etc.)."],
                VariableName = "PvPSummon",
                Default = false,
            },
            desc = MiniPet.BooleanOptionDescription,
            get = "GetGenericOption",
            set = "SetGenericOption",
        },
        [L["unsummonWhenStealthed"]] = {
            name = L["Unsummon when stealthed"],
            type = "toggle",
            width = "double",
            arg = {
                DescriptionString = L["Toggles whether companions that are learned should be automatically unsummoned when stealthing/shadowmelding/etc."],
                VariableName = "UnsummonWhenStealthed",
                Default = true,
            },
            desc = MiniPet.BooleanOptionDescription,
            get = "GetGenericOption",
            set = "SetGenericOption",
        },
        [L["pickCapturedPets"]] = {
            type = "toggle",
            order = 4,
            name = L["Pick Captured Pets"],
            width = "double",
            arg = {
                DescriptionString = L["Toggles if pets which have been captured in the wild should be picked from."],
                VariableName = "PickCapturedPets",
                Default = false,
            },
            desc = MiniPet.BooleanOptionDescription,
            get = "GetGenericOption",
            set = "SetOption_ResetPets",
        },
    },
    events = {
        "Update",
        "BANKFRAME_OPENED",
        "BANKFRAME_CLOSED",
        "BARBER_SHOP_OPEN",
        "BARBER_SHOP_CLOSE",
        "LOOT_CLOSED",
        "LOOT_OPENED",
        "UNIT_SPELLCAST_CHANNEL_START",
        "UNIT_SPELLCAST_CHANNEL_STOP",
        "UNIT_SPELLCAST_FAILED",
        "UNIT_SPELLCAST_FAILED_QUIET",
        "UNIT_SPELLCAST_INTERRUPTED",
        "UNIT_SPELLCAST_SENT",
        "UNIT_SPELLCAST_START",
        "UNIT_SPELLCAST_STOP",
        "UNIT_SPELLCAST_SUCCEEDED",
        "UPDATE_STEALTH",
        "LibLordFarlander-SpecialEvents-Aura-4.0:PlayerBuffGained",
        "LibLordFarlander-SpecialEvents-Aura-4.0:PlayerBuffLost",
    },
    -- Module variables
    isChanneling = false,
    isInDialog = false,
    isStealthed = false,
    counter = 0,
    stealthSpellIds = {
        [66] = true, -- Invisiblity
        [51753] = true, -- Camouflage
        [51755] = true, -- Camouflage
    },
    notInControlAuras = {
        [102116] = true, -- Magic Wings
    },
}

function AutoSummon:OnUPDATE_STEALTH( MiniPet )
    self.isStealthed = false
    if( IsStealthed() ) then
        self.isStealthed = true
    else
        for spellId in pairs( self.stealthSpellIds ) do
            if( SpecialEvents_Aura:UnitHasBuff( "player", spellId ) ) then
                self.isStealthed = true
                break
            end--if
        end--for        
    end--if
    if( self.isStealthed ) then
        if( MiniPet:GetSetting( moduleName, "profile", "UnsummonWhenStealthed" ) and
            MiniPet:DoCustomEventWithReturn( "HasDismissableCompanionOut" ) )  then
            MiniPet:DoCustomEvent( "DismissCompanion" )
        end--if
    end--if
end--AutoSummon:OnUPDATE_STEALTH( MiniPet )

function AutoSummon:OnUNIT_SPELLCAST_CHANNEL_START( MiniPet, unit )
    if( unit == "player" ) then
        self.isChanneling = true
    end--if
end--AutoSummon:OnUNIT_SPELLCAST_CHANNEL_START( MiniPet, unit )

function AutoSummon:OnUNIT_SPELLCAST_SENT( MiniPet, unit )
    if( unit == "player" ) then
        self.isChanneling = true
    end--if
end--AutoSummon:OnUNIT_SPELLCAST_SENT( MiniPet, unit )

function AutoSummon:OnUNIT_SPELLCAST_START( MiniPet, unit )
    if( unit == "player" ) then
        self.isChanneling = true
    end--if
end--AutoSummon:OnUNIT_SPELLCAST_START( MiniPet, unit )

function AutoSummon:OnUNIT_SPELLCAST_INTERRUPTED( MiniPet, unit )
    if( unit == "player" ) then
        self.isChanneling = false
    end--if
end--AutoSummon:OnUNIT_SPELLCAST_INTERRUPTED( MiniPet, unit )

function AutoSummon:OnUNIT_SPELLCAST_STOP( MiniPet, unit )
    if( unit == "player" ) then
        self.isChanneling = false
    end--if
end--AutoSummon:OnUNIT_SPELLCAST_STOP(( MiniPet, unit )

function AutoSummon:OnUNIT_SPELLCAST_FAILED( MiniPet, unit )
    if( unit == "player" ) then
        self.isChanneling = false
    end--if
end--AutoSummon:OnUNIT_SPELLCAST_FAILED( MiniPet, unit )

function AutoSummon:OnUNIT_SPELLCAST_FAILED_QUIET( MiniPet, unit )
    if( unit == "player" ) then
        self.isChanneling = false
    end--if
end--AutoSummon:OnUNIT_SPELLCAST_FAILED_QUIET( MiniPet, unit )

function AutoSummon:OnUNIT_SPELLCAST_SUCCEEDED( MiniPet, unit )
    if( unit == "player" ) then
        self.isChanneling = false
    end--if
end--AutoSummon:OnUNIT_SPELLCAST_SUCCEEDED( MiniPet, unit )

function AutoSummon:OnUNIT_SPELLCAST_CHANNEL_STOP( MiniPet, unit )
    if( unit == "player" ) then
        self.isChanneling = false
    end--if
end--AutoSummon:OnUNIT_SPELLCAST_CHANNEL_STOP( MiniPet, unit )

function AutoSummon:OnBANKFRAME_OPENED( MiniPet )
    self.isInDialog = true;
end--AutoSummon:OnBANKFRAME_OPENED( MiniPet )

function AutoSummon:OnBANKFRAME_CLOSED( MiniPet )
    self.isInDialog = false;
end--AutoSummon:OnBANKFRAME_CLOSED( MiniPet )

function AutoSummon:OnBARBER_SHOP_OPEN( MiniPet )
    self.isInDialog = true;
end--AutoSummon:OnBARBER_SHOP_OPEN( MiniPet )

function AutoSummon:OnBARBER_SHOP_CLOSE( MiniPet )
    self.isInDialog = false;
end--AutoSummon:OnBARBER_SHOP_CLOSE( MiniPet )

function AutoSummon:OnLOOT_OPENED( MiniPet )
    self.isInDialog = true
end--AutoSummon:OnLOOT_OPENED( MiniPet )

function AutoSummon:OnLOOT_CLOSED( MiniPet )
    self.isInDialog = false
end--AutoSummon:OnLOOT_CLOSED( MiniPet )

function AutoSummon:OnPlayerBuffGained( MiniPet, _, _, _, _, _, _, _, spellId )
    if( spellId and self.stealthSpellIds[spellId] ) then
        self.isStealthed = true
        if( MiniPet:GetSetting( moduleName, "profile", "UnsummonWhenStealthed" ) and
            MiniPet:DoCustomEventWithReturn( "HasDismissableCompanionOut" ) )  then
            MiniPet:DoCustomEvent( "DismissCompanion" )
        end--if
    end--if
end--AutoSummon:OnPlayerBuffGained( MiniPet, _, _, _, _, _, _, _, spellId )

function AutoSummon:OnPlayerBuffLost( MiniPet, _, _, _, _, spellId )
    if( spellId and self.stealthSpellIds[spellId] ) then
        self.isStealthed = IsStealthed()
    end--if
end--AutoSummon:OnPlayerBuffLost( MiniPet, _, _, _, _, spellId )

function AutoSummon:HasFullControl()
    if( HasFullControl() ) then
        return true;
    end--if
    for aura in pairs( self.notInControlAuras ) do
        if( SpecialEvents_Aura:UnitHasBuff( "player", aura ) ) then
            return false;
        end--if
    end--for
    return true;
end--

function AutoSummon:OnUpdate( MiniPet )
    if( MiniPet:GetSetting( moduleName, "profile", "AutoSummon" ) ) then
        if( not( MiniPet:DoCustomEventWithReturn( "HasDismissableCompanionOut" ) ) and
            ( not( self.IsInPvPZone() or MiniPet:GetSetting( moduleName, "profile", "PvPSummon" ) ) ) ) then
            --
            if( not( self.isChanneling ) and not( self.isInDialog ) and not( self.isStealthed ) and
                not( IsMounted() ) and not( UnitIsDeadOrGhost( "player" ) ) and
                not( UnitInVehicle( "player" ) ) and not( UnitOnTaxi( "player" ) ) and
                not( IsFalling( "player" ) ) and not( UnitAffectingCombat( "player ") ) and
                not( MiniPet:DoCustomEventWithReturn( "IsCompanionSummonInCooldown" ) ) and 
                self:HasFullControl() ) then
                --
                self.counter = self.counter + 1
                if( self.counter > 30 ) then
                    MiniPet:DoCustomEvent( "SummonCompanion" )
                    self.counter = 0
                end--if
            else
                self.counter = 0
            end--if
        else
            self.counter = 0
        end--if
    end--if
end--AutoSummon:OnUpdate( MiniPet )

function AutoSummon.IsInPvPZone()
    local _, itype = IsInInstance()
    local pvpType, isFFA = GetZonePVPInfo()

    return isFFA or ( itype == "pvp" ) or ( itype == "arena" ) or ( pvpType == "combat" )
end--AutoSummon.IsInPvPZone()

LibStub( "LibLordFarlander-Randomizer-2.0" ):RegisterModuleToHandler( "MiniPet", moduleName, AutoSummon, "Companions" )
