--[[
 Holiday Companion module for MiniPet
]]--
--[[
Copyright (c) 2011-2012, LordFarlander
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
]]--

-- $Revision: 259 $
-- $Date: 2012-09-30 17:27:43 +0000 (Sun, 30 Sep 2012) $

local L = LibStub( "AceLocale-3.0" ):GetLocale( "MiniPet holiday Module" )

local PetAndMountDatabase = LibStub( "LibLordFarlander-PetAndMountDatabase-2.0", true )
if( not PetAndMountDatabase ) then
    return
end--if

if( not LibStub( "LibLordFarlander-Randomizer-2.0", true ) ) then
    return
end--if

local TR = LibStub( "LibLordFarlander-TableRecycler-1.0", true )
if( not TR ) then
    return
end--if

local moduleName = "Holiday"

local Holiday = {
    displayname = L["Holiday"],
    settings = {
        [L["holidayOnly"]] = {
            type = "toggle",
            name = L["Force holiday only companions"],
            width = "double",
            arg = {
                DescriptionString = L["Toggles whether to only select companions which are only summonable during a holiday, when that holiday is occuring."],
                VariableName = "HolidayOnly",
                Default = false,
            },
            desc = MiniPet.BooleanOptionDescription,
            get = "GetGenericOption",
            set = "SetOption_ResetPets",
        },
    },
    events = {
        "VerifyRandom",
    },
}

function Holiday.IsHolidayToday( holiday )
    local _, _, day, _ = CalendarGetDate()

    for i = 1, CalendarGetNumDayEvents( 0, day ) do
        local title, hour, minute, calendarType, sequenceType = CalendarGetDayEvent( 0, day, i )

        if( ( calendarType == "HOLIDAY" ) and ( title == holiday ) ) then
            local gameHour, gameMinute = GetGameTime()

            if( sequenceType == "START" ) then
                -- The holiday starts today, check if now is after the start
                return ( gameHour > hour ) or ( ( gameHour == hour ) and ( gameMinute >= minute ) )
            elseif( sequenceType == "END" ) then
                -- The holiday ends today, check if now is before the end
                return ( gameHour < hour ) or ( ( gameHour == hour ) and ( gameMinute < minute ) )
            else
                -- in the middle of the holiday, it's definitely on
                return true
            end--if
        end--if
    end--for
    -- The holiday wasn't found for today
    return false
end--Holiday.IsHolidayToday( holiday )

function Holiday:OnVerifyRandom( MiniPet, itemID )
    local holiday
    local returnval = true
    local found = false
    
    if( itemID < 0 ) then
        holiday = PetAndMountDatabase:IdInSet( itemID, "items.companion.Holiday" )
    else
        holiday = PetAndMountDatabase:IdInSet( itemID * -1, "spells.companion.Holiday" )
    end--if

    if( holiday ) then
        local conditions = PetAndMountDatabase.GetConditions( holiday )

        if( conditions and conditions.Holiday ) then
            returnval = self.IsHolidayToday( L[conditions.Holiday] )
            found = true
        end--if
        TR:ReturnTable( conditions )
    end--if
    if( not( found ) and MiniPet:GetSetting( moduleName, "profile", "HolidayOnly" ) ) then
        local iterator, handle = PetAndMountDatabase:IterateSet( "items.companion.Holiday" )
        local pet, value = iterator( handle )
        local GetConditions = PetAndMountDatabase.GetConditions
        local IsHolidayToday = self.IsHolidayToday

        -- where else to build this list?
        while pet do
            local conditions = GetConditions( value )

            if( conditions and conditions.Holiday ) then
                found = IsHolidayToday( L[conditions.Holiday] )
            end--if
            TR:ReturnTable( conditions )
            if( found ) then
                break
            end--if
            
            local equipmentSlot
            
            pet, equipmentSlot = iterator( handle )
        end--for
        
        if( not found ) then
            iterator, handle = PetAndMountDatabase:IterateSet( "spells.companion.Holiday" )
            pet, value = iterator( handle )
            
            while pet do
                local conditions = GetConditions( value )

                if( conditions and conditions.Holiday ) then
                    found = IsHolidayToday( L[conditions.Holiday] )
                end--if
                TR:ReturnTable( conditions )
                if( found ) then
                    break
                end--if
                
                local equipmentSlot
                
                pet, equipmentSlot = iterator( handle )
            end--for
        end--if
        if( found ) then
            -- nope this random isn't in the holiday list
            returnval = false
        end--if
    end--if
    return returnval
end--Holiday:OnVerifyRandom( MiniPet, itemID )

LibStub( "LibLordFarlander-Randomizer-2.0" ):RegisterModuleToHandler( "MiniPet", moduleName, Holiday, "Companions" )
