--[[
    Database of all minipets and mounts
]]--
--[[
Copyright (c) 2008-2012, LordFarlander
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
]]--

-- $Revision: 262 $
-- $Date: 2012-12-02 03:02:12 +0000 (Sun, 02 Dec 2012) $

local MAJOR_VERSION = "LibLordFarlander-PetAndMountDatabase-2.0"
local MINOR_VERSION = tonumber( ( "$Revision: 262 $" ):match("(%d+)" ) ) + 90000

local LibPetAndMountDatabase, oldminor = LibStub:NewLibrary( MAJOR_VERSION, MINOR_VERSION )
if( not LibPetAndMountDatabase ) then
    return
end--if

if( not LibPetAndMountDatabase.FoundSets ) then
    LibPetAndMountDatabase.FoundSets = {}
end--if
if( not LibPetAndMountDatabase.SubSets ) then
    LibPetAndMountDatabase.SubSets = {}
end--if

local LibPeriodicTable = LibStub( "LibPeriodicTable-3.1", true )
if( not LibPeriodicTable ) then
    error( MAJOR_VERSION .. " requires LibPeriodicTable-3.1" )
end--if

local TR = LibStub( "LibLordFarlander-TableRecycler-1.0", true )
if( not TR ) then
    error( MAJOR_VERSION .. " requires LibLordFarlander-TableRecycler-1.0" )
    return
end--if

--[[
    Negative number is a SPELL instead of an ITEM
]]--

function LibPetAndMountDatabase.NormalizeSetName( set )
    if( set:find( "LibLordFarlander%-PetAndMountDatabase%-3%." ) ~= 1 ) then
        set = "LibLordFarlander-PetAndMountDatabase-3." .. set
    end--if
    return set
end--LibPetAndMountDatabase.NormalizeSetName( set )

function LibPetAndMountDatabase:HasSet( set, normalize )
    local sets = LibPeriodicTable.sets

    if( ( normalize == true ) or ( normalize == nil ) ) then
        set = self.NormalizeSetName( set )
    end--if
    if( self.FoundSets[set] ) then
        return true
    end--if
    for setName, _ in pairs( sets ) do
        if( setName:find( set ) == 1 ) then
            self.FoundSets[set] = true
            return true
        end--if
    end--if
    return false
end--LibPetAndMountDatabase:HasSet( set, normalize )

function LibPetAndMountDatabase:GetSubsets( set, normalize )
    local sets = LibPeriodicTable.sets

    if( ( normalize == true ) or ( normalize == nil ) ) then
        set = self.NormalizeSetName( set )
    end--if
    if( self.SubSets[set] ) then
        return self.SubSets[set]
    end--if
    set = set .. "."
    for setName, _ in pairs( sets ) do
        local start, ends = setName:find( set )

        if( ( start == 1 ) and ( ends ~= setName:len() ) ) then
            setName = setName:sub( ends + 1 )
            if( setName:find( "." ) ) then
                setName = ("%s%s"):format( set, strsplit( ".", setName ) )
            end--if
            if( self.SubSets[set] == nil ) then
                self.SubSets[set] = {}
            end--if
            self.SubSets[set][setName] = setName
        end--if
    end--if
    return self.SubSets[set]
end--LibPetAndMountDatabase:GetSubsets( set, normalize )

function LibPetAndMountDatabase:IdInSet( item, set, normalize )
    if( item == nil ) then
        return false
    end--if
    if( ( normalize == true ) or ( normalize == nil ) ) then
        set = self.NormalizeSetName( set )
    end--if
    return LibPeriodicTable:ItemInSet( item, set )
end--LibPetAndMountDatabase:IdInSet( item, set, normalize )


function LibPetAndMountDatabase:IterateSet( set, normalize )
    if( ( normalize == true ) or ( normalize == nil ) ) then
        set = self.NormalizeSetName( set, normalize )
    end--if
    return LibPeriodicTable:IterateSet( set )
end--LibPetAndMountDatabase:IterateSet( set, normalize )

function LibPetAndMountDatabase.GetConditions( value )
    local conditions = value:match( "%[(.-)%]" )
    local returnval

    if( conditions ) then
        conditions = TR:GetTableWithInserts( strsplit( ";", conditions ) )
        for _, condition in pairs( conditions ) do
            local name, value = condition:match( "(.-)=(.+)" )

            if( name and value ) then
                if( not returnval ) then
                    returnval = TR:GetTable()
                end--if
                returnval[name] = value
            end--if
        end--for
        TR:ReturnTable( conditions )
    end--if
    return returnval
end--if

function LibPetAndMountDatabase.GetMountAttributes( value, skipSkills, skipAttributes )
    local mountInfo = TR:GetTable()

    local groundSpeed, aquaticSpeed, flySpeed, passengers, minLevel, skills, attributes = strsplit( "|", value )
    local swimSpeed

    if( skills and not( skipSkills ) ) then
        local skillString = skills

        mountInfo.skills = TR:GetTable()
        skills = mountInfo.skills

        local skillList = TR:GetTableWithInserts( strsplit( ";", skillString ) )

        for _, skill in pairs( skillList ) do
            local skillName, value = skill:sub( 1, 1 ), tonumber( skill:sub( 2 ) )

            if( skillName == "R" ) then
                skillName = "Riding"
            elseif( skillName == "E" ) then
                skillName = "Engineering"
            elseif( skillName == "T" ) then
                skillName = "Tailoring"
            elseif( skillName == "N" ) then
                skillName = "Enchanting"
            elseif( skillName == "B" ) then
                skillName = "Blacksmithing"
            elseif( skillName == "A" ) then
                skillName = "Alchemy"
            elseif( skillName == "J" ) then
                skillName = "Jewelcrafting"
            elseif( skillName == "M" ) then
                skillName = "Mining"
            elseif( skillName == "H" ) then
                skillName = "Herbalism"
            elseif( skillName == "L" ) then
                skillName = "Leatherworking"
            elseif( skillName == "S" ) then
                skillName = "Skinning"
            elseif( skillName == "I" ) then
                skillName = "Inscription"
            elseif( skillName == "C" ) then
                skillName = "Cooking"
            elseif( skillName == "+" ) then
                skillName = "First Aid"
            elseif( skillName == "F" ) then
                skillName = "Fishing"
            elseif( skillName == "Y" ) then
                skillName = "Archeology"
            end--if
            skills[skillName] = value
        end--if
    end--if

    if( attributes and not( skipAttributes ) ) then
        local attributesString = attributes

        mountInfo.attributes = TR:GetTable()
        attributes = mountInfo.attributes

        local attributesList = TR:GetTableWithInserts( strsplit( ";", attributesString ) )

        for _, attributePair in pairs( attributesList ) do
            local attribute, value = strsplit( "=", attributePair )

            if( not( value ) or ( value == "true" ) ) then
                value = true
            elseif( value == "false" ) then
                value = false
            end--if
            attributes[attribute] = value
        end--if
    end--if

    aquaticSpeed, swimSpeed = strsplit( ";", aquaticSpeed and aquaticSpeed or "" )

    mountInfo.passengers = ( passengers and ( passengers:len() > 0 ) ) and tonumber( passengers ) or nil
    mountInfo.minLevel = ( minLevel and ( minLevel:len() > 0 ) ) and tonumber( minLevel ) or nil
    mountInfo.groundSpeed = ( groundSpeed and ( groundSpeed:len() > 0 ) ) and tonumber( groundSpeed ) or nil
    mountInfo.aquaticSpeed = ( aquaticSpeed and ( aquaticSpeed:len() > 0 ) ) and tonumber( aquaticSpeed ) or nil
    mountInfo.flySpeed = ( flySpeed and ( flySpeed:len() > 0 ) ) and tonumber( flySpeed ) or nil
    mountInfo.swimSpeed = ( swimSpeed and ( swimSpeed:len() > 0 ) ) and tonumber( swimSpeed ) or nil
    return mountInfo
end--LibPetAndMountDatabase,GetMountAttributes( value )

function LibPetAndMountDatabase.GetBattleCompanions( returnList, isWild )
    local numPets, numOwned = C_PetJournal.GetNumPets( isWild )

    for i = 1, numPets do
        local petID, speciesID, isOwned, customName, level, favorite, isRevoked, creatureName,
              icon, petType, creatureID, sourceText, description, isWildPet, canBattle = C_PetJournal.GetPetInfoByIndex( i, isWild )

        if( petID ~= "0x0000000000000000" ) then
            local fullPetID = petID

            petID = tonumber( petID:sub( 3 ), 16 )

            if( isOwned ) then
                tinsert( returnList,
                    TR:GetTable( "petID", petID,
                                 "fullPetID", fullPetID,
                                 "icon", icon,
                                 "modelID", creatureID,
                                 "name", customName or creatureName,
                                 "level", level,
                                 "speciesID", speciesID,
                                 "creatureName", creatureName,
                                 "sourceText", sourceText,
                                 "description", description,
                                 "isWildPet", isWildPet,
                                 "canBattle", canBattle  ) )
            end--if
        end--if
    end--for
end--GetBattleCompanions( returnList, isWild )

function LibPetAndMountDatabase:GetPlayersCompanions( ctype )
    local returnList = TR:GetTable()

    -- Pet battle support
    if( ctype == "BATTLEPET" ) then
        self.GetBattleCompanions( returnList, false )
        self.GetBattleCompanions( returnList, true )
    else
        for i = 1, GetNumCompanions( ctype ) do
            local creatureID, creatureName, creatureSpellID, creatureIcon, _, mountType = GetCompanionInfo( ctype, i )

            tinsert( returnList,
                TR:GetTable( "spellID", creatureSpellID,
                             "icon", creatureIcon,
                             "modelID", creatureID,
                             "name", creatureName ) )
            if( mountType ) then
                returnList[i].ground = bit.band( mountType, 0x01 ) ~= 0
                returnList[i].flying = bit.band( mountType, 0x02 ) ~= 0
                returnList[i].swimming = bit.band( mountType, 0x04 ) ~= 0
                returnList[i].underwater = bit.band( mountType, 0x08 ) ~= 0
                returnList[i].canJump =  bit.band( mountType, 0x10 ) ~= 0
            end--if
        end--for
    end--if
    return returnList
end--LibPetAndMountDatabase:GetPlayersCompanions( ctype )

function LibPetAndMountDatabase:IsCompanionType( companionName, ctype )
    for i = 1, GetNumCompanions( ctype ) do
        if( select( 2, GetCompanionInfo( ctype, i ) ) == companionName ) then
            return true
        end--if
    end--for
    return false
end--LibPetAndMountDatabase:IsCompanionType( companionName, ctype )

function LibPetAndMountDatabase:GetSummonedCompanionIndex( ctype )
    if( ctype == "BATTLEPET" ) then
        local guid = C_PetJournal.GetSummonedPetGUID()

        return guid and tonumber( guid:sub( 3 ), 16 ) or nil
    else
        for i = 1, GetNumCompanions( ctype ) do
            if( select( 5, GetCompanionInfo( ctype, i ) ) ) then
                return i
            end--if
        end--for
    end--if
    return nil
end--LibPetAndMountDatabase:GetSummonedCompanionIndex( ctype )

function LibPetAndMountDatabase:FindCompanion( ctype, spellID )
    if( spellID and ( spellID < 0 ) ) then
        spellID = spellID * -1
    end--if
    if( spellID ) then
        if( ctype == "BATTLEPET" ) then
            local isWild = false
            local numPets, numOwned = C_PetJournal.GetNumPets( isWild )

            for i = 1, numPets do
                local petID, speciesID, isOwned, customName, level, favorite, isRevoked, creatureName,
                      icon, petType, creatureID, sourceText, description, isWildPet, canBattle = C_PetJournal.GetPetInfoByIndex( i, isWild )

                petID = tonumber( petID:sub( 3 ), 16 )

                if( isOwned and ( petID == spellID ) ) then
                    return i;
                end--if
            end--for
        else
            for i = 1, GetNumCompanions( ctype ) do
                if( select( 3, GetCompanionInfo( ctype, i ) ) == spellID ) then
                    return i
                end--if
            end--for
        end--if
    end--if
    return nil
end--LibPetAndMountDatabase:FindCompanion( ctype, spellID )
