--[[
Name: LibLordFarlander-SpecialEvents-Aura-4.0
Revision: $Rev: 256 $
Author: LordFarlander
Author: Tekkub Stoutwrithe (tekkub@gmail.com)
Website: http://www.wowace.com/
Description: Special events for Auras, (de)buffs gained, lost etc.
Dependencies: LibStub, CallbackHandler-1.0, AceEvent-3.0, AceBucket-3.0, AceTimer-3.0
]]--

-- I want to rewrite this to use COMBAT_LOG_EVENT instead

--[[
Copyright (c) 2008-2011, LordFarlander
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
]]--

local vmajor, vminor = "LibLordFarlander-SpecialEvents-Aura-4.0", tonumber( ("$Revision: 256 $"):match( "(%d+)" ) ) + 90000

if( not LibStub( "LibGratuity-3.0", true ) ) then
    error( vmajor .. " requires LibGratuity-3.0" )
    return
end--if

local SpecialEvents_Aura, oldMinor = LibStub:NewLibrary( vmajor, vminor )

if( not SpecialEvents_Aura ) then
    return
end--if

local DEBUG = nil
local RL
local pairs = pairs
local string = string
local strformat = string.format
local next = next
local type = type
local UnitExists = UnitExists
local UnitIsUnit = UnitIsUnit
local GetNumRaidMembers = GetNumRaidMembers
local UnitAuraLocal = UnitAura

----------------------------
--     Initialization     --
----------------------------

local gratuity = LibStub( "LibGratuity-3.0" )

local function debug( txt )
    ChatFrame1:AddMessage( "SEEA: " .. txt )
end--debug( txt )

LibStub( "AceEvent-3.0" ):Embed( SpecialEvents_Aura )
LibStub( "AceBucket-3.0" ):Embed( SpecialEvents_Aura )
LibStub( "AceTimer-3.0" ):Embed( SpecialEvents_Aura )

if( SpecialEvents_Aura.callbacks ) then
    SpecialEvents_Aura:UnregisterAll( SpecialEvents_Aura )
else
    SpecialEvents_Aura.callbacks = LibStub( "CallbackHandler-1.0" ):New( SpecialEvents_Aura, nil, nil, "UnregisterAll" )
end--if

if( not SpecialEvents_Aura.var ) then
    SpecialEvents_Aura.vars = { buffs = {}, debuffs = {} }
end--if

--------------------------------
--      Tracking methods      --
--------------------------------

local player_last_raid_id

function SpecialEvents_Aura:PLAYER_AURAS_CHANGED()
    self:AuraScan( "player", "PLAYER_AURAS_CHANGED" )

    local id = player_last_raid_id

    if( id and UnitIsUnit( id, "player" ) ) then
        self:AuraScan( id, "PLAYER_AURAS_CHANGED Raid Player" )
    else
        for i = 1, GetNumRaidMembers() do
            id = "raid" .. i
            if( UnitIsUnit( id, "player" ) ) then
                player_last_raid_id = id
                self:AuraScan( id, "PLAYER_AURAS_CHANGED Raid Player" )
                break
            end--if
        end--for
    end--if
end--SpecialEvents_Aura:PLAYER_AURAS_CHANGED()

function SpecialEvents_Aura:PLAYER_TARGET_CHANGED()
    self:AuraScan( "target", "TARGET_CHANGED" )
    self.callbacks:Fire( "AuraTargetChanged" )
end--SpecialEvents_Aura:PLAYER_TARGET_CHANGED()

function SpecialEvents_Aura:PLAYER_FOCUS_CHANGED()
    self:AuraScan( "focus", "FOCUS_CHANGED" )
    self.callbacks:Fire( "AuraFocusChanged" )
end--SpecialEvents_Aura:PLAYER_FOCUS_CHANGED()

function SpecialEvents_Aura:PARTY_MEMBERS_CHANGED()
    -- TODO: LibRoster support?
    if( UnitExists("pet") ) then
        self:AuraScan( "pet", "PMC Pet" )
    end--if

    for i = 1, 4 do
        if( UnitExists( "party" .. i ) ) then
            self:AuraScan( "party" .. i, "PMC Party" )
        end--if
        if( UnitExists( "partypet" .. i ) ) then
            self:AuraScan( "partypet" .. i, "PMC Party Pet" )
        end--if
    end--for
    self.callbacks:Fire( "AuraPartyMembersChanged" )
end--SpecialEvents_Aura:PARTY_MEMBERS_CHANGED()

function SpecialEvents_Aura:RAID_ROSTER_UPDATE()
    -- TODO: LibRoster support?
    for i = 1, 40 do
        if( UnitExists( "raid" .. i ) ) then
            self:AuraScan( "raid" .. i, "RRU Raid" )
        end--if
        if( UnitExists( "raidpet" .. i ) ) then
            self:AuraScan( "raidpet" .. i, "RRU Raid Pet" )
        end--if
    end--for
    self.callbacks:Fire( "AuraRaidRosterUpdate" )
end--SpecialEvents_Aura:RAID_ROSTER_UPDATE()

function SpecialEvents_Aura:ScanAllAuras()
    if( UnitExists( "player" ) ) then
        self:AuraScan( "player", "ALL Player" )
    end--if
    if( UnitExists( "pet" ) ) then
        self:AuraScan( "pet", "ALL Pet" )
    end--if

    for i = 1, 4 do
        if( UnitExists( "party" .. i ) ) then
            self:AuraScan( "party" .. i, "ALL Party" )
        end--if
        if( UnitExists( "partypet" .. i ) ) then
            self:AuraScan( "partypet" .. i, "ALL Party Pet" )
        end--if
    end

    for i = 1, 40 do
        if( UnitExists( "raid" .. i ) ) then
            self:AuraScan( "raid" .. i, "ALL Raid" )
        end--if
        if( UnitExists( "raidpet" .. i ) ) then
            self:AuraScan( "raidpet" .. i, "ALL Raid Pet" )
        end--if
    end

    if( UnitExists( "target" ) ) then
        self:AuraScan( "target", "ALL Target" )
    end--if
    if( UnitExists( "focus" ) ) then
        self:AuraScan( "focus", "ALL Focus" )
    end--if
end--SpecialEvents_Aura:ScanAllAuras()

-- whee, aura scanning is fun
do
    local seenBuffs, seenDebuffs = {}, {}
    local removedBuffs = {
        spellId = {},
        name = {},
        rank = {},
        icon = {},
        count = {},
        duration = {},
        timeEnd = {},
    }
    local removedDebuffs = {
        spellId = {},
        name = {},
        rank = {},
        icon = {},
        count = {},
        dispelType = {},
        duration = {},
        timeEnd = {},
    }

    function SpecialEvents_Aura:AuraScan( unit, reason )
        if( unit == "timer" ) then
            unit, reason = "player", "Periodic for player"
        end--if

        local vars = self.vars
        local buffs, debuffs = vars.buffs[unit], vars.debuffs[unit]
        local timeNow = GetTime()

        if( not reason ) then
            reason = "UNIT_AURA[STATE]"
        end--if

        -- have we seen this unit before?
        if( not buffs ) then
            buffs = {
                index = {},
                spellId = {},
                name = {},
                rank = {},
                icon = {},
                count = {},
                duration = {},
                timeEnd = {},
            }
            vars.buffs[unit] = buffs

            debuffs = {
                index = {},
                spellId = {},
                name = {},
                rank = {},
                icon = {},
                count = {},
                dispelType = {},
                duration = {},
                timeEnd = {},
            }
            vars.debuffs[unit] = debuffs
        end--if

        --
        -- Update buffs for unit
        --

        -- check for new buffs
        local i = 1

        while true do
            local name, rank, icon, count, duration, timeLeft, timeEnd, spellId

            if( unit ) then
                local _
                
                name, rank, icon, count, _, duration, timeLeft, _, _, _, spellId = UnitAuraLocal( unit, i, "HELPFUL" )
            end--if
            if( not name ) then
                break
            end--if

            local timeEnd = ( timeLeft and (timeLeft > 0) and (timeNow + timeLeft) ) or nil

            -- buffs are the same if their name, rank, and icon are the same
            local buffIndex = strformat( "%s_%s_%s", name, rank, icon )

            -- handle multiple instances of the same buff (stacked HoTs)
            while seenBuffs[buffIndex] do
                buffIndex = buffIndex .. "_"
            end--while

            -- this is the only buff field that is allowed to change without dispatching an event
            buffs.index[buffIndex] = i

            -- new buff?
            if( not buffs.name[buffIndex] ) then
                buffs.spellId[buffIndex] = spellId
                buffs.name[buffIndex] = name
                buffs.rank[buffIndex] = rank
                buffs.icon[buffIndex] = icon
                buffs.count[buffIndex] = count
                buffs.duration[buffIndex] = duration
                buffs.timeEnd[buffIndex] = timeEnd
                seenBuffs[buffIndex] = "new"
            -- did the count change?
            elseif( buffs.count[buffIndex] ~= count ) then
                buffs.count[buffIndex] = count
                seenBuffs[buffIndex] = "changed"
            -- was the buff refreshed? only look at significant refreshes i.e. more than a second after the original buff
            elseif( timeEnd and buffs.timeEnd[buffIndex] and ( timeEnd > ( buffs.timeEnd[buffIndex] + 1 ) ) ) then
                buffs.duration[buffIndex] = duration
                buffs.timeEnd[buffIndex] = timeEnd
                seenBuffs[buffIndex] = "refreshed"
            -- no changes
            else
                seenBuffs[buffIndex] = true
            end--if

            i = i + 1
        end--while

        -- remove old buffs
        for buffIndex in pairs( buffs.index ) do
            if( not seenBuffs[buffIndex] ) then
                -- copy to removed table
                removedBuffs.spellId[buffIndex] = buffs.spellId[buffIndex]
                removedBuffs.name[buffIndex] = buffs.name[buffIndex]
                removedBuffs.rank[buffIndex] = buffs.rank[buffIndex]
                removedBuffs.icon[buffIndex] = buffs.icon[buffIndex]
                removedBuffs.count[buffIndex] = buffs.count[buffIndex]
                removedBuffs.duration[buffIndex] = buffs.duration[buffIndex]
                removedBuffs.timeEnd[buffIndex] = buffs.timeEnd[buffIndex]

                -- remove buff from unit
                buffs.spellId[buffIndex] = nil
                buffs.index[buffIndex] = nil
                buffs.name[buffIndex] = nil
                buffs.rank[buffIndex] = nil
                buffs.icon[buffIndex] = nil
                buffs.count[buffIndex] = nil
                buffs.duration[buffIndex] = nil
                buffs.timeEnd[buffIndex] = nil
            end--if
        end--for

        --
        -- Update debuffs for unit
        --

        -- check for new debuffs
        i = 1
        while true do
            local name, rank, icon, count, dispelType, duration, timeLeft, spellId

            if( unit ) then
                local _
                
                name, rank, icon, count, _, duration, timeLeft, _, _, _, spellId = UnitAuraLocal( unit, i, "HARMFUL" )
            end--if
            if( not name ) then
                break
            end--if

            local timeEnd = ( timeLeft and (timeLeft > 0) and (timeNow + timeLeft) ) or nil

            -- debuffs are the same if their name, rank, and icon are the same
            local debuffIndex = strformat( "%s_%s_%s", name, rank, icon )

            -- handle multiple instances of the same debuff
            while seenDebuffs[debuffIndex] do
                debuffIndex = debuffIndex .. "_"
            end--while

            -- these are the only debuff fields that are allowed to change without dispatching an event
            debuffs.index[debuffIndex] = i
            debuffs.dispelType[debuffIndex] = dispelType or ""

            -- new debuff?
            if( not debuffs.name[debuffIndex] ) then
                debuffs.spellId[debuffIndex] = spellId
                debuffs.name[debuffIndex] = name
                debuffs.rank[debuffIndex] = rank
                debuffs.icon[debuffIndex] = icon
                debuffs.count[debuffIndex] = count
                debuffs.duration[debuffIndex] = duration
                debuffs.timeEnd[debuffIndex] = timeEnd
                seenDebuffs[debuffIndex] = "new"
            -- did the count change?
            elseif( debuffs.count[debuffIndex] ~= count ) then
                debuffs.count[debuffIndex] = count
                seenDebuffs[debuffIndex] = "changed"
                -- was the debuff refreshed? only look at significant refreshes i.e. more than a second after the original debuff
            elseif( timeEnd and debuffs.timeEnd[debuffIndex] and ( timeEnd > ( debuffs.timeEnd[debuffIndex] + 1 ) ) ) then
                debuffs.duration[debuffIndex] = duration
                debuffs.timeEnd[debuffIndex] = timeEnd
                seenDebuffs[debuffIndex] = "refreshed"
            -- no changes
            else
                seenDebuffs[debuffIndex] = true
            end--if

            i = i + 1
        end--while

        -- remove old debuffs
        for debuffIndex in pairs( debuffs.index ) do
            if( not seenDebuffs[debuffIndex] ) then
                -- copy to removed table
                removedDebuffs.spellId[debuffIndex] =  debuffs.spellId[debuffIndex]
                removedDebuffs.name[debuffIndex] = debuffs.name[debuffIndex]
                removedDebuffs.rank[debuffIndex] = debuffs.rank[debuffIndex]
                removedDebuffs.icon[debuffIndex] = debuffs.icon[debuffIndex]
                removedDebuffs.count[debuffIndex] = debuffs.count[debuffIndex]
                removedDebuffs.dispelType[debuffIndex] = debuffs.dispelType[debuffIndex]
                removedDebuffs.duration[debuffIndex] = debuffs.duration[debuffIndex]
                removedDebuffs.timeEnd[debuffIndex] = debuffs.timeEnd[debuffIndex]

                -- remove debuff from unit
                debuffs.spellId[debuffIndex] = nil
                debuffs.index[debuffIndex] = nil
                debuffs.name[debuffIndex] = nil
                debuffs.rank[debuffIndex] = nil
                debuffs.icon[debuffIndex] = nil
                debuffs.count[debuffIndex] = nil
                debuffs.dispelType[debuffIndex] = nil
                debuffs.duration[debuffIndex] = nil
                debuffs.timeEnd[debuffIndex] = nil
            end--if
        end--for

        --
        -- Done scanning, it's time to dispatch events!
        --

        -- send events for lost buffs
        for buffIndex in pairs( removedBuffs.name ) do
            local spellId = removedBuffs.spellId[buffIndex]
            local name = removedBuffs.name[buffIndex]
            local count = removedBuffs.count[buffIndex]
            local icon = removedBuffs.icon[buffIndex]
            local rank = removedBuffs.rank[buffIndex]
            local duration = removedBuffs.duration[buffIndex]

            -- unit, name, count, icon, rank
            SpecialEvents_Aura.callbacks:Fire( "UnitBuffLost", unit, name, count, icon, rank, duration, spellId )

            if( unit == "player" ) then
                -- name, count, icon, rank
                SpecialEvents_Aura.callbacks:Fire( "PlayerBuffLost", name, count, icon, rank, spellId )
            end--if

            if( DEBUG ) then
                debug( "SpecialEvents_UnitBuffLost " .. unit .. ":" .. name .. ", " .. reason )
            end--if

            removedBuffs.name[buffIndex] = nil
            removedBuffs.rank[buffIndex] = nil
            removedBuffs.icon[buffIndex] = nil
            removedBuffs.count[buffIndex] = nil
        end--if

        -- send events for lost debuffs
        for debuffIndex in pairs( removedDebuffs.name ) do
            local spellId = removedDebuffs.spellId[debuffIndex]
            local name = removedDebuffs.name[debuffIndex]
            local count = removedDebuffs.count[debuffIndex]
            local dispelType = removedDebuffs.dispelType[debuffIndex]
            local icon = removedDebuffs.icon[debuffIndex]
            local rank = removedDebuffs.rank[debuffIndex]
            local duration = removedDebuffs.duration[debuffIndex]

            -- unit, name, count, dispelType, icon, rank
            SpecialEvents_Aura.callbacks:Fire( "UnitDebuffLost", unit, name, count, dispelType, icon, rank, duration, spellId )

            if( unit == "player" ) then
                -- name, count, dispelType, icon, rank
                SpecialEvents_Aura.callbacks:Fire( "PlayerDebuffLost", name, count, dispelType, icon, rank, spellId )
            end--if

            if( DEBUG ) then
                debug( "SpecialEvents_UnitDebuffLost " .. unit .. ":" .. name .. ", " .. reason )
            end--if

            removedDebuffs.name[debuffIndex] = nil
            removedDebuffs.rank[debuffIndex] = nil
            removedDebuffs.icon[debuffIndex] = nil
            removedDebuffs.count[debuffIndex] = nil
            removedDebuffs.dispelType[debuffIndex] = nil
        end--for

        -- send events for new/changed buffs
        for buffIndex in pairs( buffs.index ) do
            if( seenBuffs[buffIndex] == "new" ) then
                local spellId = buffs.spellId[buffIndex]
                local name = buffs.name[buffIndex]
                local index = buffs.index[buffIndex]
                local count = buffs.count[buffIndex]
                local icon = buffs.icon[buffIndex]
                local rank = buffs.rank[buffIndex]
                local duration = buffs.duration[buffIndex]
                local timeLeft = buffs.timeEnd[buffIndex] and ( buffs.timeEnd[buffIndex] - timeNow ) or nil

                -- unit, name, index, count, icon, rank
                SpecialEvents_Aura.callbacks:Fire( "UnitBuffGained", unit, name, index, count, icon, rank, duration, timeLeft, spellId )

                if( unit == "player" ) then
                    -- name, index, count, icon, rank
                    SpecialEvents_Aura.callbacks:Fire( "PlayerBuffGained", name, index, count, icon, rank, duration, timeLeft, spellId )
                end--if

                if( DEBUG ) then
                    debug( "SpecialEvents_UnitBuffGained " .. unit .. ":" .. name .. ", " .. reason .. ", " .. tostring( timeLeft ) .. "/" .. tostring( duration ) )
                end--if
            elseif( seenBuffs[buffIndex] == "changed" ) then
                local spellId = buffs.spellId[buffIndex]
                local name = buffs.name[buffIndex]
                local index = buffs.index[buffIndex]
                local count = buffs.count[buffIndex]
                local icon = buffs.icon[buffIndex]
                local rank = buffs.rank[buffIndex]
                local duration = buffs.duration[buffIndex]
                local timeLeft = buffs.timeEnd[buffIndex] and ( buffs.timeEnd[buffIndex] - timeNow ) or nil

                -- unit, name, index, count, icon, rank
                SpecialEvents_Aura.callbacks:Fire( "UnitBuffCountChanged", unit, name, index, count, icon, rank, duration, timeLeft, spellId )

                if( unit == "player" ) then
                    -- unit, name, index, count, icon, rank
                    SpecialEvents_Aura.callbacks:Fire( "PlayerBuffCountChanged", name, index, count, icon, rank, duration, timeLeft, spellId )
                end--if

                if( DEBUG ) then
                    debug( "SpecialEvents_UnitBuffCountChanged " .. unit .. ":" .. name .. ", " .. reason .. ", " .. tostring( timeLeft ) .. "/" .. tostring( duration ) )
                end--if
            elseif( seenBuffs[buffIndex] == "refreshed" ) then
                local spellId = buffs.spellId[buffIndex]
                local name = buffs.name[buffIndex]
                local index = buffs.index[buffIndex]
                local count = buffs.count[buffIndex]
                local icon = buffs.icon[buffIndex]
                local rank = buffs.rank[buffIndex]
                local duration = buffs.duration[buffIndex]
                local timeLeft = buffs.timeEnd[buffIndex] and (buffs.timeEnd[buffIndex] - timeNow) or nil

                -- unit, name, index, count, icon, rank
                SpecialEvents_Aura.callbacks:Fire( "UnitBuffRefreshed", unit, name, index, count, icon, rank, duration, timeLeft, spellId )

                if( unit == "player" ) then
                    -- name, index, count, icon, rank
                    SpecialEvents_Aura.callbacks:Fire( "PlayerBuffRefreshed", name, index, count, icon, rank, duration, timeLeft, spellId )
                end--if

                if( DEBUG ) then
                    debug( "SpecialEvents_UnitBuffRefreshed " .. unit .. ":" .. name .. ", " .. reason .. ", " .. tostring( timeLeft ) .. "/" .. tostring( duration ) )
                end--if
            end--if
        end--for

        -- send events for new/changed debuffs
        for debuffIndex in pairs( debuffs.index ) do
            if( seenDebuffs[debuffIndex] == "new" ) then
                local spellId = buffs.spellId[debuffIndex]
                local name = debuffs.name[debuffIndex]
                local count = debuffs.count[debuffIndex]
                local dispelType = debuffs.dispelType[debuffIndex]
                local icon = debuffs.icon[debuffIndex]
                local rank = debuffs.rank[debuffIndex]
                local index = debuffs.index[debuffIndex]
                local duration = debuffs.duration[debuffIndex]
                local timeLeft = debuffs.timeEnd[debuffIndex] and ( debuffs.timeEnd[debuffIndex] - timeNow ) or nil

                -- unit, name, count, dispelType, icon, rank, index
                SpecialEvents_Aura.callbacks:Fire( "UnitDebuffGained", unit, name, count, dispelType, icon, rank, index, duration, timeLeft, spellId )

                if( unit == "player" ) then
                    -- name, count, dispelType, icon, rank, index
                    SpecialEvents_Aura.callbacks:Fire( "PlayerDebuffGained", name, count, dispelType, icon, rank, index, duration, timeLeft, spellId )
                end--if

                if( DEBUG ) then
                    debug( "SpecialEvents_UnitDebuffGained " .. unit .. ":" .. name .. ", " .. reason .. ", " .. tostring( timeLeft ) .. "/" .. tostring( duration ) )
                end--if
            elseif( seenDebuffs[debuffIndex] == "changed" ) then
                local spellId = buffs.spellId[debuffIndex]
                local name = debuffs.name[debuffIndex]
                local count = debuffs.count[debuffIndex]
                local dispelType = debuffs.dispelType[debuffIndex]
                local icon = debuffs.icon[debuffIndex]
                local rank = debuffs.rank[debuffIndex]
                local index = debuffs.index[debuffIndex]
                local duration = debuffs.duration[debuffIndex]
                local timeLeft = debuffs.timeEnd[debuffIndex] and ( debuffs.timeEnd[debuffIndex] - timeNow ) or nil

                -- unit, name, count, dispelType, icon, rank, index
                SpecialEvents_Aura.callbacks:Fire( "UnitDebuffCountChanged", unit, name, count, dispelType, icon, rank, index, duration, timeLeft, spellId )

                if( unit == "player" ) then
                    -- name, count, dispelType, icon, rank, index
                    SpecialEvents_Aura.callbacks:Fire( "PlayerDebuffCountChanged", name, count, dispelType, icon, rank, index, duration, timeLeft, spellId )
                end--if

                if( DEBUG ) then
                    debug( "SpecialEvents_UnitDebuffCountChanged " .. unit .. ":" .. name .. ", " .. reason .. ", " .. tostring( timeLeft ) .. "/" .. tostring( duration ) )
                end--if
            elseif( seenDebuffs[debuffIndex] == "refreshed" ) then
                local spellId = buffs.spellId[debuffIndex]
                local name = debuffs.name[debuffIndex]
                local count = debuffs.count[debuffIndex]
                local dispelType = debuffs.dispelType[debuffIndex]
                local icon = debuffs.icon[debuffIndex]
                local rank = debuffs.rank[debuffIndex]
                local index = debuffs.index[debuffIndex]
                local duration = debuffs.duration[debuffIndex]
                local timeLeft = debuffs.timeEnd[debuffIndex] and ( debuffs.timeEnd[debuffIndex] - timeNow ) or nil

                -- unit, name, count, dispelType, icon, rank, index
                SpecialEvents_Aura.callbacks:Fire( "UnitDebuffRefreshed", unit, name, count, dispelType, icon, rank, index, duration, timeLeft, spellId )

                if( unit == "player" ) then
                    -- name, count, dispelType, icon, rank, index
                    SpecialEvents_Aura.callbacks:Fire( "PlayerDebuffRefreshed", name, count, dispelType, icon, rank, index, duration, timeLeft, spellId )
                end

                if( DEBUG ) then
                    debug( "SpecialEvents_UnitDebuffRefreshed " .. unit .. ":" .. name .. ", " .. reason .. ", " .. tostring( timeLeft ) .. "/" .. tostring( duration ) )
                end
            end
        end

        --
        -- cleanup
        --

        for k in pairs( seenBuffs ) do
            seenBuffs[k] = nil
        end--for

        for k in pairs( seenDebuffs ) do
            seenDebuffs[k] = nil
        end--for
    end--SpecialEvents_Aura:AuraScan( unit, reason )
end--do

do
    local function GetItemBuffName( slot )
        gratuity:SetInventoryItem( "player", slot )

        for i = 1, 30 do
            local text = gratuity:GetLine( i )

            if( text ) then
                local buffName = text:match( "^([^%(]+) %(%d+ [^%)]+%)$" )

                if( buffName ) then
                    return buffName
                end--if
            else
                break
            end--if
        end--if
    end--function GetItemBuffName( slot )

    function SpecialEvents_Aura:refreshItemBuffs()
        local mainhand, _, _, offhand = GetWeaponEnchantInfo()
        local vars = self.vars;

        vars.mainhandItemBuff = mainhand and GetItemBuffName( GetInventorySlotInfo( "MainHandSlot" ) )
        vars.offhandItemBuff = offhand and GetItemBuffName( GetInventorySlotInfo( "SecondaryHandSlot" ) )
    end--refreshItemBuffs()

    local first = true

    function SpecialEvents_Aura:PlayerItemBuffScan()
        if( first ) then
            self:refreshItemBuffs()
            first = false
            return
        end--if

        local vars = self.vars
        local oldMainhandItemBuff = vars.mainhandItemBuff
        local oldOffhandItemBuff = vars.offhandItemBuff

        self:refreshItemBuffs()
        if( oldMainhandItemBuff ~= vars.mainhandItemBuff ) then
            if( oldMainhandItemBuff ) then -- loss
                -- name, rank, isMainHand
                local name, rank = oldMainhandItemBuff:match( "^(.*) (%d+)$" )

                SpecialEvents_Aura.callbacks:Fire( "PlayerItemBuffLost" , name or oldMainhandItemBuff, rank, true )
            end---if
            if( vars.mainhandItemBuff ) then -- gain
                -- name, rank, isMainHand
                local name, rank = vars.mainhandItemBuff:match( "^(.*) (%d+)$" )

                SpecialEvents_Aura.callbacks:Fire( "PlayerItemBuffGained", name or vars.mainhandItemBuff, rank, true )
            end--if
        end--if
        if( oldOffhandItemBuff ~= vars.offhandItemBuff ) then
            if( oldOffhandItemBuff ) then -- loss
                -- name, rank, isMainHand
                local name, rank = oldOffhandItemBuff:match( "^(.*) (%d+)$" )

                SpecialEvents_Aura.callbacks:Fire( "PlayerItemBuffLost", name or oldOffhandItemBuff, rank, false )
            end--if
            if( vars.offhandItemBuff ) then -- gain
                -- name, rank, isMainHand
                local name, rank = vars.offhandItemBuff:match( "^(.*) (%d+)$" )

                SpecialEvents_Aura.callbacks:Fire( "PlayerItemBuffGained", name or vars.offhandItemBuff, rank, false )
            end--if
        end--if
    end--SpecialEvents_Aura:PlayerItemBuffScan()

    function SpecialEvents_Aura:GetPlayerMainHandItemBuff()
        if( self.vars.mainhandItemBuff ) then
            local name, rank = self.vars.mainhandItemBuff:match( "^(.*) (%d+)$" )

            return name or self.vars.mainhandItemBuff, rank
        end--if
    end--SpecialEvents_Aura:GetPlayerMainHandItemBuff()

    function SpecialEvents_Aura:GetPlayerOffHandItemBuff()
        if( self.vars.offhandItemBuff ) then
            local name, rank = self.vars.offhandItemBuff:match( "^(.*) (%d+)$" )

            return name or self.vars.offhandItemBuff, rank
        end--if
    end--SpecialEvents_Aura:GetPlayerOffHandItemBuff()
end--do

-----------------------------
--      Query Methods      --
-----------------------------

function SpecialEvents_Aura:UnitHasBuff( unit, name, rank, icon )
    local unitBuffs = self.vars.buffs[unit]

    if( not unitBuffs ) then
        return
    end--if

    if( ( type( name ) == "string" ) and rank and icon ) then
        local buffIndex = strformat( "%s_%s_%s", name, rank, icon )

        if( unitBuffs.index[buffIndex] ) then
            -- index; count, icon, rank
            return unitBuffs.index[buffIndex], unitBuffs.count[buffIndex], unitBuffs.icon[buffIndex], unitBuffs.rank[buffIndex], unitBuffs.spellId[buffIndex]
        else
            return
        end--if
    end--if

    for buffIndex in pairs( unitBuffs.index ) do
        if( ( ( not name ) or ( ( type( name ) == "string" ) and ( name == unitBuffs.name[buffIndex] ) or ( name == unitBuffs.spellId[buffIndex] ) ) ) and
            ( ( not rank ) or ( rank == unitBuffs.rank[buffIndex] ) ) and
            ( ( not icon ) or ( icon == unitBuffs.icon[buffIndex] ) ) ) then

            -- index; count, icon, rank
            return unitBuffs.index[buffIndex], unitBuffs.count[buffIndex], unitBuffs.icon[buffIndex], unitBuffs.rank[buffIndex]
        end--if
    end--for
end--SpecialEvents_Aura:UnitHasBuff( unit, name, rank, icon )


function SpecialEvents_Aura:UnitHasDebuff( unit, name, rank, icon )
    local unitDebuffs = self.vars.debuffs[unit]

    if( not unitDebuffs ) then
        return
    end--if

    if( ( type( name ) == "string" ) and rank and icon ) then
        local debuffIndex = strformat( "%s_%s_%s", name, rank, icon )

        if( unitDebuffs.index[debuffIndex] ) then
            -- index; count, icon, rank
            return unitDebuffs.index[debuffIndex], unitDebuffs.count[debuffIndex], unitDebuffs.icon[debuffIndex], unitDebuffs.rank[debuffIndex], unitDebuffs.spellId[debuffIndex]
        else
            return
        end--if
    end--if

    for debuffIndex, debuffName in pairs( unitDebuffs.name ) do
        if( ( ( not name ) or ( name == debuffName ) ) and
            ( ( not rank ) or ( rank == unitDebuffs.rank[debuffIndex] ) ) and
            ( ( not icon ) or ( icon == unitDebuffs.icon[debuffIndex] ) ) ) then

            -- index; count, icon, rank
            return unitDebuffs.index[debuffIndex], unitDebuffs.count[debuffIndex], unitDebuffs.icon[debuffIndex], unitDebuffs.rank[debuffIndex]
        end--if
    end--for
end--SpecialEvents_Aura:UnitHasDebuff( unit, name, rank, icon )

function SpecialEvents_Aura:UnitHasDebuffType( unit, dispelType )
    local unitDebuffs = self.vars.debuffs[unit]

    if( not unitDebuffs ) then
        return
    end--if

    for debuffIndex, debuffDispelType in pairs( unitDebuffs.dispelType ) do
        if( debuffDispelType == dispelType ) then
            -- index, count, icon, rank
            return unitDebuffs.index[debuffIndex], unitDebuffs.count[debuffIndex], unitDebuffs.icon[debuffIndex], unitDebuffs.rank[debuffIndex]
        end--if
    end--for
end--SpecialEvents_Aura:UnitHasDebuffType( unit, dispelType )

local cache = setmetatable( {}, { __mode = 'k' } )
local function donothing() end

local function iter( t )
    local unitBuffs, buffIndex = t.unitBuffs, t.buffIndex
    local index

    buffIndex, index = next( unitBuffs.index, buffIndex )
    if( buffIndex ) then
        t.buffIndex = buffIndex
        -- name, index; count, icon, rank
        return unitBuffs.name[buffIndex], index, unitBuffs.count[buffIndex], unitBuffs.icon[buffIndex], unitBuffs.rank[buffIndex], unitBuffs.duration[buffIndex], unitBuffs.spellId[buffIndex]
    else
        t.unitBuffs = nil
        t.buffIndex = nil
        cache[t] = true
    end--if
end--function iter( t )

function SpecialEvents_Aura:BuffIter( unit )
    local unitBuffs = self.vars.buffs[unit]
    local buffIndex

    if( not unitBuffs ) then
        return donothing
    end--if

    local t = next( cache ) or {}

    cache[t] = nil
    t.unitBuffs = unitBuffs

    return iter, t
end--SpecialEvents_Aura:BuffIter( unit )

local function iter( t )
    local unitDebuffs, debuffIndex = t.unitDebuffs, t.debuffIndex
    local index

    debuffIndex, index = next( unitDebuffs.index, debuffIndex )
    if( debuffIndex ) then
        t.debuffIndex = debuffIndex
        -- name, count, , icon; rank, index
        return unitDebuffs.name[debuffIndex], unitDebuffs.count[debuffIndex], unitDebuffs.dispelType[debuffIndex], unitDebuffs.icon[debuffIndex], unitDebuffs.rank[debuffIndex], index, unitDebuffs.duration[debuffIndex], unitDebuffs.spellId[debuffIndex]
    else
        t.unitDebuffs = nil
        t.debuffIndex = nil
        cache[t] = true
    end--if
end--function iter( t )

function SpecialEvents_Aura:DebuffIter( unit )
    local unitDebuffs = self.vars.debuffs[unit]
    local debuffIndex, index

    if( not unitDebuffs ) then
        return donothing
    end

    local t = next( cache ) or {}

    cache[t] = nil
    t.unitDebuffs = unitDebuffs

    return iter, t
end--SpecialEvents_Aura:DebuffIter( unit )

SpecialEvents_Aura:RegisterEvent( "UNIT_AURA", "AuraScan")
SpecialEvents_Aura:RegisterEvent( "UNIT_AURASTATE", "AuraScan" )
SpecialEvents_Aura:RegisterBucketEvent( "PLAYER_AURAS_CHANGED", 0.2 ); -- bucketed
SpecialEvents_Aura:RegisterEvent( "PLAYER_TARGET_CHANGED" );
SpecialEvents_Aura:RegisterEvent( "PLAYER_FOCUS_CHANGED" )
-- scan the player auras every second; there is no event fired when a player's buff is refreshed by another player, so period scan is necessary
SpecialEvents_Aura:ScheduleRepeatingTimer( "AuraScan", 1, "timer" )
-- scan the player item buffs every 0.2 seconds; there is no event that fires when these are added or removed.
SpecialEvents_Aura:ScheduleRepeatingTimer( "PlayerItemBuffScan", 0.2 )
-- TODO: LibRoster support
SpecialEvents_Aura:RegisterBucketEvent( "PARTY_MEMBERS_CHANGED", 0.2 ); -- bucketed
SpecialEvents_Aura:RegisterBucketEvent( "RAID_ROSTER_UPDATE", 0.2 ); -- bucketed

SpecialEvents_Aura:RegisterEvent( "PLAYER_ENTERING_WORLD", "ScanAllAuras" )
